@@ -1,11 +1,14 @@
 #!/usr/bin/env python
-import xml.etree.ElementTree as ET
+try:
+    import xml.etree.cElementTree as ET
+except ImportError:
+    import xml.etree.ElementTree as ET
 from libnmap.objects import NmapHost, NmapService, NmapReport
 
 
 class NmapParser(object):
     @classmethod
-    def parse(cls, nmap_data=None, data_type='XML'):
+    def parse(cls, nmap_data=None, data_type='XML', incomplete=False):
         """
             Generic class method of NmapParser class.
             The data to be parsed does not need to be a complete nmap
@@ -22,6 +25,11 @@ class NmapParser(object):
             :param data_type: specifies the type of data to be parsed.
             :type data_type: string ("XML"|"JSON"|"YAML").
 
+            :param incomplete: enable you to parse interrupted nmap scans
+            and/or incomplete nmap xml blocks by adding a </nmaprun> at
+            the end of the scan.
+            :type incomplete: boolean
+
             As of today, only XML parsing is supported.
 
             :return: NmapObject (NmapHost, NmapService or NmapReport)
@@ -29,7 +37,7 @@ class NmapParser(object):
 
         nmapobj = None
         if data_type == "XML":
-            nmapobj = cls._parse_xml(nmap_data)
+            nmapobj = cls._parse_xml(nmap_data, incomplete)
         else:
             raise NmapParserException("Unknown data type provided. "
                                       "Please check documentation for "
@@ -37,7 +45,7 @@ class NmapParser(object):
         return nmapobj
 
     @classmethod
-    def _parse_xml(cls, nmap_data=None):
+    def _parse_xml(cls, nmap_data=None, incomplete=False):
         """
             Protected class method used to process a specific data type.
             In this case: XML. This method is called by cls.parse class
@@ -58,6 +66,11 @@ class NmapParser(object):
                 4. a list of hosts: <hosts/> tag (TODO)
                 5. a list of ports: <ports/> tag
 
+            :param incomplete: enable you to parse interrupted nmap scans
+            and/or incomplete nmap xml blocks by adding a </nmaprun> at
+            the end of the scan.
+
+            :type incomplete: boolean
             :return: NmapObject (NmapHost, NmapService or NmapReport)
                     or a list of NmapObject
         """
@@ -69,6 +82,9 @@ class NmapParser(object):
             raise NmapParserException("wrong nmap_data type given as "
                                       "argument: cannot parse data")
 
+        if incomplete is True:
+            nmap_data += "</nmaprun>"
+
         try:
             root = ET.fromstring(nmap_data)
         except:
@@ -115,14 +131,14 @@ class NmapParser(object):
                 nmap_scan['_hosts'].append(cls._parse_xml_host(el))
             elif el.tag == 'runstats':
                 nmap_scan['_runstats'] = cls.__parse_runstats(el)
-            #else:
+            # else:
             #    print "struct pparse unknown attr: {0} value: {1}".format(
             #        el.tag,
             #        el.get(el.tag))
         return NmapReport(nmap_scan)
 
     @classmethod
-    def parse_fromstring(cls, nmap_data, data_type="XML"):
+    def parse_fromstring(cls, nmap_data, data_type="XML", incomplete=False):
         """
             Call generic cls.parse() method and ensure that a string is
             passed on as argument. If not, an exception is raised.
@@ -135,16 +151,23 @@ class NmapParser(object):
 
             :param data_type: Specifies the type of data passed on as argument.
 
+            :param incomplete: enable you to parse interrupted nmap scans
+            and/or incomplete nmap xml blocks by adding a </nmaprun> at
+            the end of the scan.
+            :type incomplete: boolean
+
             :return: NmapObject
         """
 
         if not isinstance(nmap_data, str):
             raise NmapParserException("bad argument type for "
                                       "xarse_fromstring(): should be a string")
-        return cls.parse(nmap_data, data_type)
+        return cls.parse(nmap_data, data_type, incomplete)
 
     @classmethod
-    def parse_fromfile(cls, nmap_report_path, data_type="XML"):
+    def parse_fromfile(cls, nmap_report_path,
+                       data_type="XML",
+                       incomplete=False):
         """
             Call generic cls.parse() method and ensure that a correct file
             path is given as argument. If not, an exception is raised.
@@ -158,13 +181,18 @@ class NmapParser(object):
 
             :param data_type: Specifies the type of serialization in the file.
 
+            :param incomplete: enable you to parse interrupted nmap scans
+            and/or incomplete nmap xml blocks by adding a </nmaprun> at
+            the end of the scan.
+            :type incomplete: boolean
+
             :return: NmapObject
         """
 
         try:
             with open(nmap_report_path, 'r') as fileobj:
                 fdata = fileobj.read()
-                rval = cls.parse(fdata, data_type)
+                rval = cls.parse(fdata, data_type, incomplete)
         except IOError:
             raise
         return rval
@@ -204,6 +232,7 @@ class NmapParser(object):
                               address=h['__NmapHost__']['_address'],
                               status=h['__NmapHost__']['_status'],
                               hostnames=h['__NmapHost__']['_hostnames'],
+                              extras=h['__NmapHost__']['_extras'],
                               services=slist)
                 hlist.append(nh)
             nreport['_hosts'] = hlist
@@ -243,7 +272,7 @@ class NmapParser(object):
         _hostnames = []
         _services = []
         _status = {}
-        _address = {}
+        _addresses = []
         _host_extras = {}
         extra_tags = ['uptime', 'distance', 'tcpsequence',
                       'ipidsequence', 'tcptssequence', 'times']
@@ -252,18 +281,23 @@ class NmapParser(object):
                 for hostname in cls.__parse_hostnames(xh):
                     _hostnames.append(hostname)
             elif xh.tag == 'ports':
-                for port in cls._parse_xml_ports(xh):
+                ports_dict = cls._parse_xml_ports(xh)
+                for port in ports_dict['ports']:
                     _services.append(port)
+                _host_extras['extraports'] = ports_dict['extraports']
             elif xh.tag == 'status':
                 _status = cls.__format_attributes(xh)
             elif xh.tag == 'address':
-                _address = cls.__format_attributes(xh)
+                _addresses.append(cls.__format_attributes(xh))
             elif xh.tag == 'os':
                 _os_extra = cls.__parse_os_fingerprint(xh)
                 _host_extras.update({'os': _os_extra})
+            elif xh.tag == 'hostscript':
+                _host_scripts = cls.__parse_host_scripts(xh)
+                _host_extras.update({'hostscript': _host_scripts})
             elif xh.tag in extra_tags:
                 _host_extras[xh.tag] = cls.__format_attributes(xh)
-            #else:
+            # else:
             #    print "struct host unknown attr: %s value: %s" %
             #           (h.tag, h.get(h.tag))
         _stime = ''
@@ -274,7 +308,7 @@ class NmapParser(object):
             _etime = _host_header['endtime']
         nhost = NmapHost(_stime,
                          _etime,
-                         _address,
+                         _addresses,
                          _status,
                          _hostnames,
                          _services,
@@ -316,15 +350,18 @@ class NmapParser(object):
 
         xelement = cls.__format_element(scanports_data)
 
-        ports = []
+        rdict = {'ports': [], 'extraports': None}
         for xservice in xelement:
             if xservice.tag == 'port':
                 nport = cls._parse_xml_port(xservice)
-                ports.append(nport)
-            #else:
+                rdict['ports'].append(nport)
+            elif xservice.tag == 'extraports':
+                extraports = cls.__parse_extraports(xservice)
+                rdict['extraports'] = extraports
+            # else:
             #    print "struct port unknown attr: %s value: %s" %
             #           (h.tag, h.get(h.tag))
-        return ports
+        return rdict
 
     @classmethod
     def _parse_xml_port(cls, scanport_data):
@@ -349,15 +386,17 @@ class NmapParser(object):
 
         _state = None
         _service = None
-        _service_extras = []
+        _service_scripts = []
+        _service_extras = {}
         for xport in xelement:
             if xport.tag == 'state':
                 _state = cls.__format_attributes(xport)
             elif xport.tag == 'service':
                 _service = cls.__format_attributes(xport)
             elif xport.tag == 'script':
-                _script_dict = cls.__format_attributes(xport)
-                _service_extras.append(_script_dict)
+                _script_dict = cls.__parse_script(xport)
+                _service_scripts.append(_script_dict)
+        _service_extras['scripts'] = _service_scripts
 
         if(_portid is None or _protocol is None
                 or _state is None or _service is None):
@@ -372,6 +411,77 @@ class NmapParser(object):
                             _service_extras)
         return nport
 
+    @classmethod
+    def __parse_extraports(cls, extraports_data):
+        """
+            Private method parsing the data from extra scanned ports.
+            X extraports were in state "closed" server returned "conn-refused"
+            tag: <extraports>
+
+            :param extraports_data: XML data for extraports
+            :type extraports_data: xml.ElementTree.Element or a string
+
+            :return: python dict with following keys: state, count, reason
+        """
+        rdict = {'state': '', 'count': '', 'reasons': []}
+        xelement = cls.__format_element(extraports_data)
+        extraports_dict = cls.__format_attributes(xelement)
+
+        if 'state' in extraports_dict:
+            rdict['state'] = extraports_dict
+        if 'count' in extraports_dict:
+            rdict['count'] = extraports_dict
+        for xelt in xelement:
+            if xelt.tag == 'extrareasons':
+                extrareasons_dict = cls.__format_attributes(xelt)
+                rdict['reasons'].append(extrareasons_dict)
+        return rdict
+
+    @classmethod
+    def __parse_script(cls, script_data):
+        """
+            Private method parsing the data from NSE scripts output
+
+            :param script_data: portion of XML describing the results of the
+            script data
+            :type script_data: xml.ElementTree.Element or a string
+
+            :return: python dict holding scripts output
+        """
+        _script_dict = cls.__format_attributes(script_data)
+
+        _elt_dict = {}
+        for script_elem in script_data:
+            if script_elem.tag == 'elem':
+                _elt_dict.update({script_elem.get('key'): script_elem.text})
+            elif script_elem.tag == 'table':
+                tdict = {}
+                for telem in script_elem:
+                    tdict[telem.get('key')] = telem.text
+                _elt_dict[script_elem.get('key')] = tdict
+        _script_dict['elements'] = _elt_dict
+        return _script_dict
+
+    @classmethod
+    def __parse_host_scripts(cls, scripts_data):
+        """
+            Private method parsing the data from scripts affecting
+            the target host.
+            Contents of <hostscript> is returned as a list of dict.
+
+            :param scripts_data: portion of XML describing the results of the
+            scripts data
+            :type scripts_data: xml.ElementTree.Element or a string
+
+            :return: python list holding scripts output in a dict
+        """
+        _host_scripts = []
+        for xscript in scripts_data:
+            if xscript.tag == 'script':
+                _script_dict = cls.__parse_script(xscript)
+            _host_scripts.append(_script_dict)
+        return _host_scripts
+
     @classmethod
     def __parse_os_fingerprint(cls, os_data):
         """
@@ -407,7 +517,10 @@ class NmapParser(object):
         rdict['osmatch'] = os_match_probability
         rdict['osclass'] = os_class_probability
         rdict['ports_used'] = os_ports_used
-        rdict['osfingerprint'] = os_fp['fingerprint']
+        if 'fingerprint' in os_fp:
+            rdict['osfingerprint'] = os_fp['fingerprint']
+        else:
+            rdict['osfingerprint'] = ''
 
         return rdict
 
