@@ -6,6 +6,7 @@ import subprocess
 import threading
 from threading import Thread
 from xml.dom import pulldom
+import warnings
 try:
     from Queue import Queue, Empty, Full
 except ImportError:
@@ -66,7 +67,7 @@ class NmapProcess(Thread):
             self.__nmap_targets = targets
         else:
             raise Exception("Supplied target list should be either a "
-                            "string of a list")
+                            "string or a list")
 
         self._nmap_options = set(options.split())
         if safe_mode and not self._nmap_options.isdisjoint(unsafe_opts):
@@ -85,6 +86,8 @@ class NmapProcess(Thread):
         self.__io_queue = Queue()
         self.__ioerr_queue = Queue()
         self.__process_killed = threading.Event()
+        self.__thread_stdout = None
+        self.__thread_stderr = None
 
         # API usable in callback function
         self.__state = self.READY
@@ -97,7 +100,6 @@ class NmapProcess(Thread):
         self.__summary = ''
         self.__stdout = ''
         self.__stderr = ''
-        self.initial_threads = 0
 
     def _run_init(self):
         """
@@ -181,6 +183,34 @@ class NmapProcess(Thread):
 
         return rc
 
+    def sudo_run_background(self, run_as='root'):
+        """
+        Public method enabling the library's user to run in background a
+        nmap scan with priviledges via sudo.
+        The sudo configuration should be set manually on the local system
+        otherwise sudo will prompt for a password.
+        This method alters the command line by prefixing the sudo command to
+        nmap and will then call self.run()
+
+        :param run_as: user name to which the lib needs to sudo to run the scan
+
+        :return: return code from nmap execution
+        """
+        sudo_user = run_as.split().pop()
+        try:
+            pwd.getpwnam(sudo_user).pw_uid
+        except KeyError:
+            raise
+
+        sudo_path = self._whereis("sudo")
+        if sudo_path is None:
+            raise EnvironmentError(2, "sudo is not installed or "
+                                      "could not be found in system path: "
+                                      "cannot run nmap with sudo")
+
+        self.__sudo_run = "{0} -u {1}".format(sudo_path, sudo_user)
+        super(NmapProcess, self).start()
+
     def run(self):
         """
         Public method which is usually called right after the constructor
@@ -215,13 +245,18 @@ class NmapProcess(Thread):
                                                 stdout=subprocess.PIPE,
                                                 stderr=subprocess.PIPE,
                                                 bufsize=0)
-            self.initial_threads = threading.active_count()
-            Thread(target=stream_reader, name='stdout-reader',
-                   args=(self.__nmap_proc.stdout,
-                         self.__io_queue)).start()
-            Thread(target=stream_reader, name='stderr-reader',
-                   args=(self.__nmap_proc.stderr,
-                         self.__ioerr_queue)).start()
+            self.__thread_stdout = Thread(target=stream_reader,
+                                          name='stdout-reader',
+                                          args=(self.__nmap_proc.stdout,
+                                                self.__io_queue))
+            self.__thread_stderr = Thread(target=stream_reader,
+                                          name='stderr-reader',
+                                          args=(self.__nmap_proc.stderr,
+                                                self.__ioerr_queue))
+
+            self.__thread_stdout.start()
+            self.__thread_stderr.start()
+
             self.__state = self.RUNNING
         except OSError:
             self.__state = self.FAILED
@@ -229,6 +264,10 @@ class NmapProcess(Thread):
 
         return self.__wait()
 
+    def active_fd(self):
+        return (self.__thread_stdout.is_alive() or
+                self.__thread_stderr.is_alive())
+
     def __wait(self):
         """
         Private method, called by run() which will loop and
@@ -243,10 +282,13 @@ class NmapProcess(Thread):
         """
         thread_stream = ''
         while (self.__nmap_proc.poll() is None or
-               threading.active_count() != self.initial_threads or
-               not self.__io_queue.empty()):
+               self.active_fd() is True or
+               not self.__io_queue.empty() or
+               not self.__ioerr_queue.empty()):
             if self.__process_killed.isSet():
                 break
+            if not self.__ioerr_queue.empty():
+                self.__stderr += self.__ioerr_queue.get_nowait()
             try:
                 thread_stream = self.__io_queue.get_nowait()
             except Empty:
@@ -269,14 +311,13 @@ class NmapProcess(Thread):
         else:
             self.__state = self.FAILED
             try:
-                self.__stderr = self.__ioerr_queue.get(timeout=1)
+                self.__stderr += self.__ioerr_queue.get(timeout=1)
                 self.__ioerr_queue.task_done()
             except Empty:
                 pass
         return self.rc
 
     def run_background(self):
-        self.daemon = True
         super(NmapProcess, self).start()
 
     def is_running(self):
@@ -413,6 +454,8 @@ class NmapProcess(Thread):
 
         :return: string. Unix timestamp
         """
+        warnings.warn("data collected from finished events are deprecated."
+                      "Use NmapParser.parse()", DeprecationWarning)
         return self.__endtime
 
     @property
@@ -422,6 +465,8 @@ class NmapProcess(Thread):
 
         :return: string
         """
+        warnings.warn("data collected from finished events are deprecated."
+                      "Use NmapParser.parse()", DeprecationWarning)
         return self.__elapsed
 
     @property
@@ -431,6 +476,8 @@ class NmapProcess(Thread):
 
         :return: string
         """
+        warnings.warn("data collected from finished events are deprecated."
+                      "Use NmapParser.parse()", DeprecationWarning)
         return self.__summary
 
     @property
